#if defined(ARCHIMEDES_C)
#include <io51.h>
#endif

#if defined(SDCC)
#include <at89x051.h>
#endif

#include <stdio.h>


/*code char *COPYR = "FREQMETER 2004 (c) 3Z6AEF";*/


typedef unsigned char byte;
typedef unsigned int  word;
typedef unsigned long dword;
#define _NOP _asm nop _endasm


#if defined(ARCHIMEDES_C)
#define TIMER1_TIME (0xffff-1000-20)   /* 1 milisek*/
#endif

#if defined(SDCC)
#define TIMER1_TIME (0xffff-960)   /* 1 milisek*/
#endif


volatile byte Timer0_Cnt;
volatile byte Timer1_Cnt;


/*
//-------------------------------------- 7-seg display
//     a
//     -
//  f | | b
//	g  -
//  e | | c
//	d  _ . h (point)
//
*/

#define seg_a 0x10
#define seg_b 0x40
#define seg_c 0x02
#define seg_d 0x01
#define seg_e 0x08
#define seg_f 0x20
#define seg_g 0x80
#define seg_h 0x04
#define DecimalPoint seg_h

#define ch_blank 0
#define ch_0    ( seg_a | seg_b | seg_c | seg_d | seg_e | seg_f  )
#define ch_1    ( seg_b | seg_c  )
#define ch_2    ( seg_a | seg_b | seg_g | seg_e | seg_d  )
#define ch_3    ( seg_a | seg_b | seg_d | seg_c | seg_g  )
#define ch_4    ( seg_f | seg_g | seg_b | seg_c  )
#define ch_5    ( seg_a | seg_f | seg_g | seg_c | seg_d  )
#define ch_6    ( seg_a | seg_f | seg_e | seg_d | seg_c | seg_g  )
#define ch_7    ( seg_a | seg_b | seg_c  )
#define ch_8    ( seg_a | seg_b  | seg_c | seg_d | seg_e | seg_f | seg_g  )
#define ch_9    ( seg_a | seg_b  | seg_c | seg_d | seg_f | seg_g  )
#define ch_A    ( seg_a | seg_b | seg_c | seg_e | seg_f | seg_g )
#define ch_B    ( seg_f | seg_e | seg_c | seg_d | seg_g )
#define ch_C    ( seg_a | seg_f | seg_e | seg_d )
#define ch_D    ( seg_b | seg_c | seg_d | seg_e | seg_g )
#define ch_E    ( seg_a | seg_f | seg_e | seg_d | seg_g )
#define ch_F    ( seg_a | seg_f | seg_e | seg_g )
#define ch_H    ( seg_b | seg_c | seg_e | seg_f | seg_g )
#define ch_U    ( seg_f | seg_e | seg_d | seg_c | seg_b ) 
#define ch_S    ( seg_a | seg_f | seg_g | seg_c | seg_d )
#define ch_L    ( seg_f | seg_e | seg_d  ) 
#define ch_P    ( seg_a | seg_b | seg_g | seg_f | seg_e )
#define ch_r    ( seg_e | seg_g )
#define ch_n    ( seg_e | seg_g | seg_c )

/*
// Binary to 7-segment decoder 
// D0..D6 -> A..F, D7-point
*/
code byte Bin2Seg[16] = { 
  ch_0, ch_1, ch_2, ch_3, ch_4, ch_5, ch_6, ch_7, 
  ch_8, ch_9, ch_A, ch_B, ch_C, ch_D, ch_E, ch_blank
};

#define MAXDISPNUM 4
volatile char DispVal[MAXDISPNUM];
volatile byte DispNum;  /* current position displayed: 0..(MAXDISPNUM-1)*/

/*volatile dword FreqCnt;*/
volatile word FreqCnt;
volatile bit FreqCntRdy;

code byte DispPos[MAXDISPNUM] = { 
  0xfe, 0xfd, 0xfb, 0xf7
};


#if defined(ARCHIMEDES_C)
interrupt [0xb] void Timer0_ISR(void) 
#endif
#if defined(SDCC)
void Timer0_ISR(void) interrupt TF0_VECTOR _naked
#endif
{
  ++Timer0_Cnt;
#if defined(SDCC)
  _asm	reti  _endasm;
#endif
}/*Timer0_ISR*/


#if defined(ARCHIMEDES_C)
interrupt [0x1b] void Timer1_ISR(void) 
#endif
#if defined(SDCC)
void Timer1_ISR(void) interrupt TF1_VECTOR 
#endif
{
  byte i;

  TR1= 0;
  TH1= (byte)((TIMER1_TIME)>>8);
  TL1= (byte)(TIMER1_TIME);
  TR1= 1;

  /*-------------------------------------*/
  ++Timer1_Cnt;
  if(Timer1_Cnt>=100) {
    Timer1_Cnt=0;
    
    TR0= 0; 
    FreqCnt= (((dword)TH0)<<8) | ((dword)TL0) ;   /* f/16*/
    TH0= 0;
    TL0= 0;
    TR0= 1; 
    FreqCntRdy=1;
  }


  /*----------------------------- DispLED*/
  i = Bin2Seg[DispVal[DispNum]]; 
  if(DispNum==3) i |= DecimalPoint; /* decimal points fixed on 3 position ( "3.500")*/
  P3 = 0xff; /* all diplays off*/
  P1 = ~i;
  P3= DispPos[DispNum];
  if(++DispNum>=MAXDISPNUM) DispNum=0;

}/*Timer1_ISR*/



void InitTimers(void)
{
  Timer0_Cnt=0;
  Timer1_Cnt=0;
  FreqCnt=0;
  FreqCntRdy=0;

  /*---------------- INIT*/
  TCON=0;
  /* Timer0: 16-bit Counter, Timer1: 16 bit Timer*/
  TMOD = 0x15; /* 00010101*/
  TH1 = (byte)((TIMER1_TIME)>>8);
  TL1 = (byte)(TIMER1_TIME);
  TH0=0;
  TL0=0;
  /* Run Timer0 and Timer1*/
  TCON = 0x50; /* 01010000  */
  IE=0x88; /* 10001000 Timer1 overflow interrupt enabled*/

}/*InitTimers*/


#define IF_FREQ 8000

void main(void)
{
  byte i;
  dword xx;
  word yy;

  IP=0;
  InitTimers();

  for(;;) {

    if(FreqCntRdy) {
      FreqCntRdy=0;

      xx = FreqCnt;
      xx <<= 5;
      xx /= 100L;
      yy = IF_FREQ - (xx - 4);

      for(i=0; i<MAXDISPNUM; ++i) {
	DispVal[i] = yy%10;
	yy /= 10;
      }

      InitTimers();

    }
  }
  

}/*main*/



